@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Lang = imports.lang;
 const DBus = imports.dbus;
@@ -23,8 +23,6 @@ const AutorunSetting = {
     ASK: 3
 };
 
-const HOTPLUG_ICON_SIZE = 16;
-
 // misc utils
 function ignoreAutorunForMount(mount) {
     let root = mount.get_root();
@@ -279,12 +277,12 @@ AutorunResidentSource.prototype = {
     __proto__: MessageTray.Source.prototype,
 
     _init: function() {
-        MessageTray.Source.prototype._init.call(this, _('Removable Devices'));
+        MessageTray.Source.prototype._init.call(this, _("Removable Devices"));
 
         this._mounts = [];
 
         this._notification = new AutorunResidentNotification(this);
-        this._setSummaryIcon(this.createNotificationIcon(HOTPLUG_ICON_SIZE));
+        this._setSummaryIcon(this.createNotificationIcon());
     },
 
     addMount: function(mount, apps) {
@@ -329,10 +327,10 @@ AutorunResidentSource.prototype = {
         }
     },
 
-    createNotificationIcon: function(iconSize) {
+    createNotificationIcon: function() {
         return new St.Icon ({ icon_name: 'media-removable',
                               icon_type: St.IconType.FULLCOLOR,
-                              icon_size: iconSize ? iconSize : this.ICON_SIZE });
+                              icon_size: this.ICON_SIZE });
     }
 }
 
@@ -493,7 +491,7 @@ AutorunTransientDispatcher.prototype = {
         let app = null;
 
         if (setting == AutorunSetting.RUN) {
-            app = Gio.app_info_get_default_for_type(type, false);
+            app = Gio.app_info_get_default_for_type(contentTypes[0], false);
         } else if (setting == AutorunSetting.FILES) {
             app = Gio.app_info_get_default_for_type('inode/directory', false);
         }
@@ -533,16 +531,16 @@ AutorunTransientSource.prototype = {
         this.apps = apps;
 
         this._notification = new AutorunTransientNotification(this);
-        this._setSummaryIcon(this.createNotificationIcon(this.ICON_SIZE));
+        this._setSummaryIcon(this.createNotificationIcon());
 
         // add ourselves as a source, and popup the notification
         Main.messageTray.add(this);
         this.notify(this._notification);
     },
 
-    createNotificationIcon: function(iconSize) {
+    createNotificationIcon: function() {
         return new St.Icon({ gicon: this.mount.get_icon(),
-                             icon_size: iconSize ? iconSize : this.ICON_SIZE });
+                             icon_size: this.ICON_SIZE });
     }
 }
 
