@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
@@ -14,12 +14,12 @@ const St = imports.gi.St;
 
 const AutomountManager = imports.ui.automountManager;
 const AutorunManager = imports.ui.autorunManager;
-const Chrome = imports.ui.chrome;
 const CtrlAltTab = imports.ui.ctrlAltTab;
 const EndSessionDialog = imports.ui.endSessionDialog;
 const PolkitAuthenticationAgent = imports.ui.polkitAuthenticationAgent;
 const Environment = imports.ui.environment;
 const ExtensionSystem = imports.ui.extensionSystem;
+const Keyboard = imports.ui.keyboard;
 const MessageTray = imports.ui.messageTray;
 const Overview = imports.ui.overview;
 const Panel = imports.ui.panel;
@@ -27,6 +27,7 @@ const PlaceDisplay = imports.ui.placeDisplay;
 const RunDialog = imports.ui.runDialog;
 const Layout = imports.ui.layout;
 const LookingGlass = imports.ui.lookingGlass;
+const NetworkAgent = imports.ui.networkAgent;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const Scripting = imports.ui.scripting;
@@ -43,7 +44,6 @@ DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
 
 let automountManager = null;
 let autorunManager = null;
-let chrome = null;
 let panel = null;
 let hotCorners = [];
 let placesManager = null;
@@ -64,14 +64,92 @@ let uiGroup = null;
 let magnifier = null;
 let xdndHandler = null;
 let statusIconDispatcher = null;
+let keyboard = null;
 let layoutManager = null;
+let networkAgent = null;
 let _errorLogStack = [];
 let _startDate;
 let _defaultCssStylesheet = null;
 let _cssStylesheet = null;
+let _gdmCssStylesheet = null;
 
 let background = null;
 
+function _createUserSession() {
+    // Load the calendar server. Note that we are careful about
+    // not loading any events until the user presses the clock
+    global.launch_calendar_server();
+
+    placesManager = new PlaceDisplay.PlacesManager();
+    telepathyClient = new TelepathyClient.Client();
+    automountManager = new AutomountManager.AutomountManager();
+    autorunManager = new AutorunManager.AutorunManager();
+    networkAgent = new NetworkAgent.NetworkAgent();
+}
+
+function _createGDMSession() {
+    // We do this this here instead of at the top to prevent GDM
+    // related code from getting loaded in normal user sessions
+    const LoginDialog = imports.gdm.loginDialog;
+
+    let loginDialog = new LoginDialog.LoginDialog();
+    loginDialog.connect('loaded', function() {
+                            loginDialog.open();
+                        });
+}
+
+function _initRecorder() {
+    let recorderSettings = new Gio.Settings({ schema: 'org.gnome.shell.recorder' });
+
+    global.screen.connect('toggle-recording', function() {
+        if (recorder == null) {
+            recorder = new Shell.Recorder({ stage: global.stage });
+        }
+
+        if (recorder.is_recording()) {
+            recorder.pause();
+            Meta.enable_unredirect_for_screen(global.screen);
+        } else {
+            // read the parameters from GSettings always in case they have changed
+            recorder.set_framerate(recorderSettings.get_int('framerate'));
+            recorder.set_filename('shell-%d%u-%c.' + recorderSettings.get_string('file-extension'));
+            let pipeline = recorderSettings.get_string('pipeline');
+
+            if (!pipeline.match(/^\s*$/))
+                recorder.set_pipeline(pipeline);
+            else
+                recorder.set_pipeline(null);
+
+            Meta.disable_unredirect_for_screen(global.screen);
+            recorder.record();
+        }
+    });
+}
+
+function _initUserSession() {
+    _initRecorder();
+
+    global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT, false, -1, 1);
+
+    ExtensionSystem.init();
+    ExtensionSystem.loadExtensions();
+
+    let shellwm = global.window_manager;
+
+    shellwm.takeover_keybinding('panel_run_dialog');
+    shellwm.connect('keybinding::panel_run_dialog', function () {
+       getRunDialog().open();
+    });
+
+    shellwm.takeover_keybinding('panel_main_menu');
+    shellwm.connect('keybinding::panel_main_menu', function () {
+        overview.toggle();
+    });
+
+    global.display.connect('overlay-key', Lang.bind(overview, overview.toggle));
+
+}
+
 function start() {
     // Monkey patch utility functions into the global proxy;
     // This is easier and faster than indirecting down into global
@@ -91,10 +169,6 @@ function start() {
     // back into sync ones.
     DBus.session.flush();
 
-    // Load the calendar server. Note that we are careful about
-    // not loading any events until the user presses the clock
-    global.launch_calendar_server();
-
     // Ensure ShellWindowTracker and ShellAppUsage are initialized; this will
     // also initialize ShellAppSystem first.  ShellAppSystem
     // needs to load all the .desktop files, and ShellWindowTracker
@@ -113,74 +187,51 @@ function start() {
     global.stage.no_clear_hint = true;
 
     _defaultCssStylesheet = global.datadir + '/theme/gnome-shell.css';
+    _gdmCssStylesheet = global.datadir + '/theme/gdm.css';
     loadTheme();
 
-    let shellwm = global.window_manager;
-    shellwm.takeover_keybinding('panel_main_menu');
-    shellwm.connect('keybinding::panel_main_menu', function () {
-        overview.toggle();
-    });
-    shellwm.takeover_keybinding('panel_run_dialog');
-    shellwm.connect('keybinding::panel_run_dialog', function () {
-       getRunDialog().open();
-    });
-
     // Set up stage hierarchy to group all UI actors under one container.
-    uiGroup = new Clutter.Group();
+    uiGroup = new Shell.GenericContainer({ name: 'uiGroup' });
+    uiGroup.connect('allocate',
+                    function (actor, box, flags) {
+                        let children = uiGroup.get_children();
+                        for (let i = 0; i < children.length; i++)
+                            children[i].allocate_preferred_size(flags);
+                    });
     St.set_ui_root(global.stage, uiGroup);
     global.window_group.reparent(uiGroup);
     global.overlay_group.reparent(uiGroup);
     global.stage.add_actor(uiGroup);
 
     layoutManager = new Layout.LayoutManager();
-    placesManager = new PlaceDisplay.PlacesManager();
     xdndHandler = new XdndHandler.XdndHandler();
     ctrlAltTabManager = new CtrlAltTab.CtrlAltTabManager();
-    overview = new Overview.Overview();
-    chrome = new Chrome.Chrome();
+    // This overview object is just a stub for non-user sessions
+    overview = new Overview.Overview({ isDummy: global.session_type != Shell.SessionType.USER });
     magnifier = new Magnifier.Magnifier();
     statusIconDispatcher = new StatusIconDispatcher.StatusIconDispatcher();
     panel = new Panel.Panel();
     wm = new WindowManager.WindowManager();
     messageTray = new MessageTray.MessageTray();
+    keyboard = new Keyboard.Keyboard();
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
-    telepathyClient = new TelepathyClient.Client();
-    automountManager = new AutomountManager.AutomountManager();
-    autorunManager = new AutorunManager.AutorunManager();
+
+    if (global.session_type == Shell.SessionType.USER)
+        _createUserSession();
+    else if (global.session_type == Shell.SessionType.GDM)
+        _createGDMSession();
+
+    panel.startStatusArea();
 
     layoutManager.init();
+    keyboard.init();
     overview.init();
+
+    if (global.session_type == Shell.SessionType.USER)
+        _initUserSession();
     statusIconDispatcher.start(messageTray.actor);
 
-    _startDate = new Date();
-
-    let recorderSettings = new Gio.Settings({ schema: 'org.gnome.shell.recorder' });
-
-    global.screen.connect('toggle-recording', function() {
-        if (recorder == null) {
-            recorder = new Shell.Recorder({ stage: global.stage });
-        }
-
-        if (recorder.is_recording()) {
-            recorder.pause();
-        } else {
-            // read the parameters from GSettings always in case they have changed
-            recorder.set_framerate(recorderSettings.get_int('framerate'));
-            recorder.set_filename('shell-%d%u-%c.' + recorderSettings.get_string('file-extension'));
-            let pipeline = recorderSettings.get_string('pipeline');
-
-            if (!pipeline.match(/^\s*$/))
-                recorder.set_pipeline(pipeline);
-            else
-                recorder.set_pipeline(null);
-
-            recorder.record();
-        }
-    });
-
-    global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT, false, -1, 1);
-
     // Provide the bus object for gnome-session to
     // initiate logouts.
     EndSessionDialog.init();
@@ -188,14 +239,7 @@ function start() {
     // Attempt to become a PolicyKit authentication agent
     PolkitAuthenticationAgent.init()
 
-    ExtensionSystem.init();
-    ExtensionSystem.loadExtensions();
-
-    panel.startStatusArea();
-    panel.startupAnimation();
-
-    let display = global.screen.get_display();
-    display.connect('overlay-key', Lang.bind(overview, overview.toggle));
+    _startDate = new Date();
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
@@ -414,6 +458,9 @@ function loadTheme() {
 
     let theme = new St.Theme ({ application_stylesheet: cssStylesheet });
 
+    if (global.session_type == Shell.SessionType.GDM)
+        theme.load_stylesheet(_gdmCssStylesheet);
+
     if (previousTheme) {
         let customStylesheets = previousTheme.get_custom_stylesheets();
 
@@ -424,6 +471,19 @@ function loadTheme() {
     themeContext.set_theme (theme);
 }
 
+/**
+ * notify:
+ * @msg: A message
+ * @details: Additional information
+ */
+function notify(msg, details) {
+    let source = new MessageTray.SystemNotificationSource();
+    messageTray.add(source);
+    let notification = new MessageTray.Notification(source, msg, details);
+    notification.setTransient(true);
+    source.notify(notification);
+}
+
 /**
  * notifyError:
  * @msg: An error message
@@ -434,15 +494,11 @@ function loadTheme() {
 function notifyError(msg, details) {
     // Also print to stderr so it's logged somewhere
     if (details)
-        log("error: " + msg + ": " + details);
+        log('error: ' + msg + ': ' + details);
     else
-        log("error: " + msg)
+        log('error: ' + msg);
 
-    let source = new MessageTray.SystemNotificationSource();
-    messageTray.add(source);
-    let notification = new MessageTray.Notification(source, msg, details);
-    notification.setTransient(true);
-    source.notify(notification);
+    notify(msg, details);
 }
 
 /**
@@ -524,9 +580,8 @@ function _globalKeyPressHandler(actor, event) {
     let keyCode = event.get_key_code();
     let modifierState = Shell.get_event_state(event);
 
-    let display = global.screen.get_display();
     // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
-    let action = display.get_keybinding_action(keyCode, modifierState);
+    let action = global.display.get_keybinding_action(keyCode, modifierState);
 
     // The screenshot action should always be available (even if a
     // modal dialog is present)
@@ -538,9 +593,9 @@ function _globalKeyPressHandler(actor, event) {
         return true;
     }
 
-    // Other bindings are only available when the overview is up and
+    // Other bindings are only available to the user session when the overview is up and
     // no modal dialog is present.
-    if (!overview.visible || modalCount > 1)
+    if (global.session_type == Shell.SessionType.USER && (!overview.visible || modalCount > 1))
         return false;
 
     // This isn't a Meta.KeyBindingAction yet
@@ -549,6 +604,16 @@ function _globalKeyPressHandler(actor, event) {
         return true;
     }
 
+    if (action == Meta.KeyBindingAction.SWITCH_PANELS) {
+        ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK,
+                                modifierState);
+        return true;
+    }
+
+    // None of the other bindings are relevant outside of the user's session
+    if (global.session_type != Shell.SessionType.USER)
+        return false;
+
     switch (action) {
         // left/right would effectively act as synonyms for up/down if we enabled them;
         // but that could be considered confusing; we also disable them in the main view.
@@ -572,9 +637,6 @@ function _globalKeyPressHandler(actor, event) {
         case Meta.KeyBindingAction.PANEL_MAIN_MENU:
             overview.hide();
             return true;
-        case Meta.KeyBindingAction.SWITCH_PANELS:
-            ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK);
-            return true;
     }
 
     return false;
@@ -702,7 +764,6 @@ function popModal(actor, timestamp) {
 function createLookingGlass() {
     if (lookingGlass == null) {
         lookingGlass = new LookingGlass.LookingGlass();
-        lookingGlass.slaveTo(panel.actor);
     }
     return lookingGlass;
 }
