@@ -1,4 +1,4 @@
-// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 const ByteArray = imports.byteArray;
 const DBus = imports.dbus;
 const GLib = imports.gi.GLib;
@@ -1291,6 +1291,9 @@ NMDeviceWireless.prototype = {
         apObj.accessPoints.splice(i, 1);
 
         if (apObj.accessPoints.length == 0) {
+            if (this._activeNetwork == apObj)
+                this._activeNetwork = null;
+
             if (apObj.item)
                 apObj.item.destroy();
 
@@ -1480,6 +1483,11 @@ NMDeviceWireless.prototype = {
     },
 
     _createNetworkItem: function(apObj, position) {
+        if(!apObj.accessPoints || apObj.accessPoints.length == 0) {
+            // this should not happen, but I have no idea why it happens
+            return;
+        }
+
         if(apObj.connections.length > 0) {
             if (apObj.connections.length == 1)
                 apObj.item = this._createAPItem(apObj.connections[0], apObj, false);
@@ -1742,13 +1750,6 @@ NMApplet.prototype = {
         if (wrapperClass) {
             let wrapper = new wrapperClass(this._client, device, this._connections);
 
-            wrapper._networkLostId = wrapper.connect('network-lost', Lang.bind(this, function(device) {
-                this._notifyForDevice(device, 'network-offline',
-                                      _("Connectivity lost"),
-                                      _("You're no longer connected to the network"),
-                                      // set critical urgency to popup the notification automatically
-                                      MessageTray.Urgency.CRITICAL);
-            }));
             wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(device, reason) {
                 // XXX: nm-applet has no special text depending on reason
                 // but I'm not sure of this generic message
@@ -1761,7 +1762,6 @@ NMApplet.prototype = {
                 this._syncSectionTitle(dev.category);
             }));
             wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
-                wrapper.disconnect(wrapper._networkLostId);
                 wrapper.disconnect(wrapper._activationFailedId);
                 wrapper.disconnect(wrapper._deviceStateChangedId);
                 wrapper.disconnect(wrapper._destroyId);
@@ -1828,7 +1828,7 @@ NMApplet.prototype = {
             if (!a._inited) {
                 a._notifyDefaultId = a.connect('notify::default', Lang.bind(this, this._updateIcon));
                 a._notifyDefault6Id = a.connect('notify::default6', Lang.bind(this, this._updateIcon));
-                a._notifyStateId = a.connect('notify::state', Lang.bind(this, this._updateIcon));
+                a._notifyStateId = a.connect('notify::state', Lang.bind(this, this._notifyActivated));
 
                 a._inited = true;
             }
@@ -1872,12 +1872,28 @@ NMApplet.prototype = {
 
                 if (a._primaryDevice)
                     a._primaryDevice.setActiveConnection(a);
+
+                if (a.state == NetworkManager.ActiveConnectionState.ACTIVATED
+                    && a._primaryDevice && a._primaryDevice._notification) {
+                    a._primaryDevice._notification.destroy();
+                    a._primaryDevice._notification = null;
+                }
             }
         }
 
         this._mainConnection = activating || default_ip4 || default_ip6 || this._activeConnections[0] || null;
     },
 
+    _notifyActivated: function(activeConnection) {
+        if (activeConnection.state == NetworkManager.ActiveConnectionState.ACTIVATED
+            && activeConnection._primaryDevice && activeConnection._primaryDevice._notification) {
+            activeConnection._primaryDevice._notification.destroy();
+            activeConnection._primaryDevice._notification = null;
+        }
+
+        this._updateIcon();
+    },
+
     _readConnections: function() {
         let connections = this._settings.list_connections();
         for (let i = 0; i < connections.length; i++) {
