@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const GConf = imports.gi.GConf;
@@ -94,27 +94,52 @@ function WindowClone(realWindow) {
 
 WindowClone.prototype = {
     _init : function(realWindow) {
-        this.actor = new Clutter.Clone({ source: realWindow.get_texture(),
-                                         reactive: true,
-                                         x: realWindow.x,
-                                         y: realWindow.y });
-        this.actor._delegate = this;
         this.realWindow = realWindow;
         this.metaWindow = realWindow.meta_window;
         this.metaWindow._delegate = this;
-        this.origX = realWindow.x;
-        this.origY = realWindow.y;
+
+        let [borderX, borderY] = this._getInvisibleBorderPadding();
+        this._windowClone = new Clutter.Clone({ source: realWindow.get_texture(),
+                                                x: -borderX,
+                                                y: -borderY });
+        // We expect this.actor to be used for all interaction rather than
+        // this._windowClone; as the former is reactive and the latter
+        // is not, this just works for most cases. However, for DND all
+        // actors are picked, so DND operations would operate on the clone.
+        // To avoid this, we hide it from pick.
+        Shell.util_set_hidden_from_pick(this._windowClone, true);
+
+        this.origX = realWindow.x + borderX;
+        this.origY = realWindow.y + borderY;
+
+        let outerRect = realWindow.meta_window.get_outer_rect();
+
+        // The MetaShapedTexture that we clone has a size that includes
+        // the invisible border; this is inconvenient; rather than trying
+        // to compensate all over the place we insert a ClutterGroup into
+        // the hierarchy that is sized to only the visible portion.
+        this.actor = new Clutter.Group({ reactive: true,
+                                         x: this.origX,
+                                         y: this.origY,
+                                         width: outerRect.width,
+                                         height: outerRect.height });
+
+        this.actor.add_actor(this._windowClone);
+
+        this.actor._delegate = this;
 
         this._stackAbove = null;
 
-        this._sizeChangedId = this.realWindow.connect('size-changed', Lang.bind(this, function() {
-            this.emit('size-changed');
-        }));
+        this._sizeChangedId = this.realWindow.connect('size-changed',
+            Lang.bind(this, this._onRealWindowSizeChanged));
         this._realWindowDestroyId = this.realWindow.connect('destroy',
             Lang.bind(this, this._disconnectRealWindowSignals));
 
-        this.actor.connect('button-release-event',
-                           Lang.bind(this, this._onButtonRelease));
+        let clickAction = new Clutter.ClickAction();
+        clickAction.connect('clicked', Lang.bind(this, this._onClicked));
+        clickAction.connect('long-press', Lang.bind(this, this._onLongPress));
+
+        this.actor.add_action(clickAction);
 
         this.actor.connect('scroll-event',
                            Lang.bind(this, this._onScroll));
@@ -125,6 +150,7 @@ WindowClone.prototype = {
 
         this._draggable = DND.makeDraggable(this.actor,
                                             { restoreOnSuccess: true,
+                                              manualMode: true,
                                               dragActorMaxSize: WINDOW_DND_SIZE,
                                               dragActorOpacity: DRAGGING_WINDOW_OPACITY });
         this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
@@ -161,7 +187,7 @@ WindowClone.prototype = {
             // will look funny.
 
             if (!this._selected &&
-                this.metaWindow != global.screen.get_display().focus_window)
+                this.metaWindow != global.display.focus_window)
                 this._zoomEnd();
         }
     },
@@ -176,6 +202,32 @@ WindowClone.prototype = {
         this._realWindowDestroyId = 0;
     },
 
+    _getInvisibleBorderPadding: function() {
+        // We need to adjust the position of the actor because of the
+        // consequences of invisible borders -- in reality, the texture
+        // has an extra set of "padding" around it that we need to trim
+        // down.
+
+        // The outer rect paradoxically is the smaller rectangle,
+        // containing the positions of the visible frame. The input
+        // rect contains everything, including the invisible border
+        // padding.
+        let outerRect = this.metaWindow.get_outer_rect();
+        let inputRect = this.metaWindow.get_input_rect();
+        let [borderX, borderY] = [outerRect.x - inputRect.x,
+                                  outerRect.y - inputRect.y];
+
+        return [borderX, borderY];
+    },
+
+    _onRealWindowSizeChanged: function() {
+        let [borderX, borderY] = this._getInvisibleBorderPadding();
+        let outerRect = this.metaWindow.get_outer_rect();
+        this.actor.set_size(outerRect.width, outerRect.height);
+        this._windowClone.set_position(-borderX, -borderY);
+        this.emit('size-changed');
+    },
+
     _onDestroy: function() {
         this._disconnectRealWindowSignals();
 
@@ -297,9 +349,27 @@ WindowClone.prototype = {
         this._zoomStep           = undefined;
     },
 
-    _onButtonRelease : function (actor, event) {
+    _onClicked: function(action, actor) {
         this._selected = true;
-        this.emit('selected', event.get_time());
+        this.emit('selected', global.get_current_time());
+    },
+
+    _onLongPress: function(action, actor, state) {
+        // Take advantage of the Clutter policy to consider
+        // a long-press canceled when the pointer movement
+        // exceeds dnd-drag-threshold to manually start the drag
+        if (state == Clutter.LongPressState.CANCEL) {
+            // A click cancels a long-press before any click handler is
+            // run - make sure to not start a drag in that case
+            Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this,
+                function() {
+                    if (this._selected)
+                        return;
+                    let [x, y] = action.get_coords();
+                    this._draggable.startDrag(x, y, global.get_current_time());
+                }));
+        }
+        return true;
     },
 
     _onDragBegin : function (draggable, time) {
@@ -1440,7 +1510,7 @@ Workspace.prototype = {
                                                  time);
             return true;
         } else if (source.shellWorkspaceLaunch) {
-            source.shellWorkspaceLaunch({ workspace: this.metaWorkspace,
+            source.shellWorkspaceLaunch({ workspace: this.metaWorkspace ? this.metaWorkspace.index() : -1,
                                           timestamp: time });
             return true;
         }
