@@ -19,9 +19,7 @@
 typedef enum {
   MATCH_NONE,
   MATCH_SUBSTRING, /* Not prefix, substring */
-  MATCH_MULTIPLE_SUBSTRING, /* Matches multiple criteria with substrings */
   MATCH_PREFIX, /* Strict prefix */
-  MATCH_MULTIPLE_PREFIX, /* Matches multiple criteria, at least one prefix */
 } ShellAppSearchMatch;
 
 /* This is mainly a memory usage optimization - the user is going to
@@ -1255,20 +1253,21 @@ _shell_app_match_search_terms (ShellApp  *app,
       current_match = MATCH_NONE;
 
       p = strstr (app->casefolded_name, term);
-      if (p == app->casefolded_name)
-        current_match = MATCH_PREFIX;
-      else if (p != NULL)
-        current_match = MATCH_SUBSTRING;
+      if (p != NULL)
+        {
+          if (p == app->casefolded_name || *(p - 1) == ' ')
+            current_match = MATCH_PREFIX;
+          else
+            current_match = MATCH_SUBSTRING;
+        }
 
       p = strstr (app->casefolded_exec, term);
       if (p != NULL)
         {
-          if (p == app->casefolded_exec)
-            current_match = (current_match == MATCH_NONE) ? MATCH_PREFIX
-                                                          : MATCH_MULTIPLE_PREFIX;
+          if (p == app->casefolded_exec || *(p - 1) == '-')
+            current_match = MATCH_PREFIX;
           else if (current_match < MATCH_PREFIX)
-            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
-                                                          : MATCH_MULTIPLE_SUBSTRING;
+            current_match = MATCH_SUBSTRING;
         }
 
       if (app->casefolded_description && current_match < MATCH_PREFIX)
@@ -1278,8 +1277,7 @@ _shell_app_match_search_terms (ShellApp  *app,
            */
           p = strstr (app->casefolded_description, term);
           if (p != NULL)
-            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
-                                                          : MATCH_MULTIPLE_SUBSTRING;
+            current_match = MATCH_SUBSTRING;
         }
 
       if (current_match == MATCH_NONE)
@@ -1294,9 +1292,7 @@ _shell_app_match_search_terms (ShellApp  *app,
 void
 _shell_app_do_match (ShellApp         *app,
                      GSList           *terms,
-                     GSList          **multiple_prefix_results,
                      GSList          **prefix_results,
-                     GSList          **multiple_substring_results,
                      GSList          **substring_results)
 {
   ShellAppSearchMatch match;
@@ -1317,15 +1313,9 @@ _shell_app_do_match (ShellApp         *app,
     {
       case MATCH_NONE:
         break;
-      case MATCH_MULTIPLE_PREFIX:
-        *multiple_prefix_results = g_slist_prepend (*multiple_prefix_results, app);
-        break;
       case MATCH_PREFIX:
         *prefix_results = g_slist_prepend (*prefix_results, app);
         break;
-      case MATCH_MULTIPLE_SUBSTRING:
-        *multiple_substring_results = g_slist_prepend (*multiple_substring_results, app);
-        break;
       case MATCH_SUBSTRING:
         *substring_results = g_slist_prepend (*substring_results, app);
         break;
@@ -1369,6 +1359,7 @@ shell_app_finalize (GObject *object)
   g_free (app->casefolded_name);
   g_free (app->name_collation_key);
   g_free (app->casefolded_description);
+  g_free (app->casefolded_exec);
 
   G_OBJECT_CLASS(shell_app_parent_class)->finalize (object);
 }
